@@ -1,7 +1,6 @@
-from __future__ import absolute_import
+from abc import ABC, abstractmethod
 from threading import Lock
 
-from sentry_sdk._compat import iteritems
 from sentry_sdk._types import TYPE_CHECKING
 from sentry_sdk.utils import logger
 
@@ -70,21 +69,36 @@ _DEFAULT_INTEGRATIONS = [
 
 _AUTO_ENABLING_INTEGRATIONS = [
     "sentry_sdk.integrations.aiohttp.AioHttpIntegration",
+    "sentry_sdk.integrations.ariadne.AriadneIntegration",
+    "sentry_sdk.integrations.arq.ArqIntegration",
+    "sentry_sdk.integrations.asyncpg.AsyncPGIntegration",
     "sentry_sdk.integrations.boto3.Boto3Integration",
     "sentry_sdk.integrations.bottle.BottleIntegration",
     "sentry_sdk.integrations.celery.CeleryIntegration",
+    "sentry_sdk.integrations.chalice.ChaliceIntegration",
+    "sentry_sdk.integrations.clickhouse_driver.ClickhouseDriverIntegration",
     "sentry_sdk.integrations.django.DjangoIntegration",
     "sentry_sdk.integrations.falcon.FalconIntegration",
     "sentry_sdk.integrations.fastapi.FastApiIntegration",
     "sentry_sdk.integrations.flask.FlaskIntegration",
+    "sentry_sdk.integrations.gql.GQLIntegration",
+    "sentry_sdk.integrations.graphene.GrapheneIntegration",
     "sentry_sdk.integrations.httpx.HttpxIntegration",
+    "sentry_sdk.integrations.huey.HueyIntegration",
+    "sentry_sdk.integrations.huggingface_hub.HuggingfaceHubIntegration",
+    "sentry_sdk.integrations.langchain.LangchainIntegration",
+    "sentry_sdk.integrations.loguru.LoguruIntegration",
     "sentry_sdk.integrations.openai.OpenAIIntegration",
+    "sentry_sdk.integrations.pymongo.PyMongoIntegration",
     "sentry_sdk.integrations.pyramid.PyramidIntegration",
+    "sentry_sdk.integrations.quart.QuartIntegration",
     "sentry_sdk.integrations.redis.RedisIntegration",
     "sentry_sdk.integrations.rq.RqIntegration",
     "sentry_sdk.integrations.sanic.SanicIntegration",
     "sentry_sdk.integrations.sqlalchemy.SqlalchemyIntegration",
     "sentry_sdk.integrations.starlette.StarletteIntegration",
+    "sentry_sdk.integrations.starlite.StarliteIntegration",
+    "sentry_sdk.integrations.strawberry.StrawberryIntegration",
     "sentry_sdk.integrations.tornado.TornadoIntegration",
 ]
 
@@ -125,7 +139,7 @@ def setup_integrations(
                 integrations[instance.identifier] = instance
                 used_as_default_integration.add(instance.identifier)
 
-    for identifier, integration in iteritems(integrations):
+    for identifier, integration in integrations.items():
         with _installer_lock:
             if identifier not in _processed_integrations:
                 logger.debug(
@@ -133,16 +147,6 @@ def setup_integrations(
                 )
                 try:
                     type(integration).setup_once()
-                except NotImplementedError:
-                    if getattr(integration, "install", None) is not None:
-                        logger.warning(
-                            "Integration %s: The install method is "
-                            "deprecated. Use `setup_once`.",
-                            identifier,
-                        )
-                        integration.install()
-                    else:
-                        raise
                 except DidNotEnable as e:
                     if identifier not in used_as_default_integration:
                         raise
@@ -157,7 +161,7 @@ def setup_integrations(
 
     integrations = {
         identifier: integration
-        for identifier, integration in iteritems(integrations)
+        for identifier, integration in integrations.items()
         if identifier in _installed_integrations
     }
 
@@ -177,7 +181,7 @@ class DidNotEnable(Exception):  # noqa: N818
     """
 
 
-class Integration(object):
+class Integration(ABC):
     """Baseclass for all integrations.
 
     To accept options for an integration, implement your own constructor that
@@ -191,6 +195,7 @@ class Integration(object):
     """String unique ID of integration type"""
 
     @staticmethod
+    @abstractmethod
     def setup_once():
         # type: () -> None
         """
@@ -203,4 +208,4 @@ class Integration(object):
         Inside those hooks `Integration.current` can be used to access the
         instance again.
         """
-        raise NotImplementedError()
+        pass
