@@ -1,10 +1,10 @@
-from sentry_sdk import Hub
+import sentry_sdk
 from sentry_sdk.consts import OP, SPANDATA
-from sentry_sdk.hub import _should_send_default_pii
 from sentry_sdk.integrations import Integration, DidNotEnable
 from sentry_sdk.tracing import Span
 from sentry_sdk._types import TYPE_CHECKING
-from sentry_sdk.utils import capture_internal_exceptions
+from sentry_sdk.scope import should_send_default_pii
+from sentry_sdk.utils import capture_internal_exceptions, ensure_integration_enabled
 
 from typing import TypeVar
 
@@ -74,16 +74,14 @@ T = TypeVar("T")
 
 
 def _wrap_start(f: Callable[P, T]) -> Callable[P, T]:
+    @ensure_integration_enabled(ClickhouseDriverIntegration, f)
     def _inner(*args: P.args, **kwargs: P.kwargs) -> T:
-        hub = Hub.current
-        if hub.get_integration(ClickhouseDriverIntegration) is None:
-            return f(*args, **kwargs)
         connection = args[0]
         query = args[1]
         query_id = args[2] if len(args) > 2 else kwargs.get("query_id")
         params = args[3] if len(args) > 3 else kwargs.get("params")
 
-        span = hub.start_span(op=OP.DB, description=query)
+        span = sentry_sdk.start_span(op=OP.DB, description=query)
 
         connection._sentry_span = span  # type: ignore[attr-defined]
 
@@ -94,7 +92,7 @@ def _wrap_start(f: Callable[P, T]) -> Callable[P, T]:
         if query_id:
             span.set_data("db.query_id", query_id)
 
-        if params and _should_send_default_pii():
+        if params and should_send_default_pii():
             span.set_data("db.params", params)
 
         # run the original code
@@ -112,11 +110,11 @@ def _wrap_end(f: Callable[P, T]) -> Callable[P, T]:
         span = instance.connection._sentry_span  # type: ignore[attr-defined]
 
         if span is not None:
-            if res is not None and _should_send_default_pii():
+            if res is not None and should_send_default_pii():
                 span.set_data("db.result", res)
 
             with capture_internal_exceptions():
-                span.hub.add_breadcrumb(
+                span.scope.add_breadcrumb(
                     message=span._data.pop("query"), category="query", data=span._data
                 )
 
@@ -135,7 +133,7 @@ def _wrap_send_data(f: Callable[P, T]) -> Callable[P, T]:
 
         _set_db_data(span, instance.connection)
 
-        if _should_send_default_pii():
+        if should_send_default_pii():
             db_params = span._data.get("db.params", [])
             db_params.extend(data)
             span.set_data("db.params", db_params)
